//csv module

/*
 This module provides the low-level underpinnings of a CSV reading/writing
 module.  Users should not use this module directly, but import the csv.py
 module instead.
 */
package org.python.modules.csv;

import org.python.core.ArgParser;
import org.python.core.Py;
import org.python.core.PyClass;
import org.python.core.PyException;
import org.python.core.PyFieldDescr;
import org.python.core.PyInstance;
import org.python.core.PyInteger;
import org.python.core.PyIterator;
import org.python.core.PyList;
import org.python.core.PyNewWrapper;
import org.python.core.PyNone;
import org.python.core.PyObject;
import org.python.core.PyString;
import org.python.core.PyType;
import org.python.expose.ExposedType;
import org.python.expose.ExposedGet;
import org.python.expose.ExposedSet;
import org.python.expose.ExposedNew;



// struct DialectObj (_csv.c)
@ExposedType(name = "Dialect")
public class PyDialect extends PyObject {
    public String __doc__ = "CSV dialect\n" +
        "\n" +
        "The Dialect type records CSV parsing and generation options.\n";

    // is " represented by ""?
    @ExposedGet
    @ExposedSet
    public boolean doublequote;

    // field separator
    @ExposedGet
    @ExposedSet
    public char delimiter;

    // quote character
    @ExposedGet
    @ExposedSet
    public char quotechar;

    // escape character
    @ExposedGet
    @ExposedSet
    public char escapechar;

    // ignore spaces following delimiter?
    @ExposedGet
    @ExposedSet
    public boolean skipinitialspace;

    // string to write between records
    @ExposedGet
    @ExposedSet
    public PyObject lineterminator;

    // style of quoting to write
    @ExposedGet
    @ExposedSet
    public int quoting;

    // raise exception on bad CSV
    @ExposedGet
    @ExposedSet
    public boolean strict;



/*
    public static void typeSetup(PyObject dict, PyType.Newstyle marker) {
        dict.__setitem__("__new__", new PyNewWrapper(PyObject.class,
                    "__new__", -1, -1) {
            public PyObject new_impl(boolean init, PyType subtype,
                PyObject[] args, String[] kws) {
                return new PyDialect(args, kws);
            }
        });

        dict.__setitem__("doublequote", new PyFieldDescr("doublequote",
                    PyDialect.class, "doublequote"));
        dict.__setitem__("delimiter", new PyFieldDescr("delimiter",
                    PyDialect.class, "delimiter"));
        dict.__setitem__("quotechar", new PyFieldDescr("quotechar",
                    PyDialect.class, "quotechar"));
        dict.__setitem__("escapechar", new PyFieldDescr("escapechar",
                    PyDialect.class, "escapechar"));
        dict.__setitem__("skipinitialspace", new PyFieldDescr(
                    "skipinitialspace",
                    PyDialect.class, "skipinitialspace"));
        dict.__setitem__("lineterminator", new PyFieldDescr(
                    "lineterminator",
                    PyDialect.class, "lineterminator"));
        dict.__setitem__("quoting", new PyFieldDescr("quoting",
                    PyDialect.class, "quoting"));
        dict.__setitem__("strict", new PyFieldDescr("strict",
                    PyDialect.class, "strict"));
    }
*/

    public void __delattr__(String name) {
        throw Py.TypeError("Cannot delete numeric/char attribute");
    }

    public PyObject __findattr__(String name) {
        // instead of get_None_as_nullchar in _csv.c
        if ("escapechar".equals(name) && escapechar == '\0') {
            return Py.None;
        }
        return super.__findattr__(name);
    }

    public void __setattr__(String name, PyObject value) {
        if ("lineterminator".equals(name)) {
            // instead of set_string in _csv.c
            if (!(value instanceof PyString)) {
                throw Py
                    .TypeError("Bad argument type for built-in operation");
            }
        } else if ("escapechar".equals(name)) {
            // instead of set_None_as_nullchar in _csv.c
            if (value instanceof PyNone) {
                escapechar = '\0';
                return;
            } else if (value instanceof PyString) {
                PyString str = (PyString) value;
                if (str.__len__() != 1) {
                    throw Py.TypeError("Bad argument");
                }
            } else {
                throw Py.TypeError("Bad argument");
            }
        } else if ("quoting".equals(name)) {
            if (!(value instanceof PyInteger)) {
                throw Py.TypeError("quoting must be integer (0/1)");
            }
            PyInteger num_obj = (PyInteger) value;
            int num = num_obj.getValue();
            if(num < _csv.QUOTE_MINIMAL || num > _csv.QUOTE_NONE) {
                throw Py.TypeError("bad argument type for built-in operation");
            }
        }
        super.__setattr__(name, value);
    }

    @ExposedNew
    final static PyObject local___new__(PyNewWrapper new_,
                                        boolean init,
                                        PyType subtype,
                                        PyObject[] args,
                                        String[] keywords) {
        System.out.println("(PyDialect.local___new__) starting");
        PyDialect newobj;
        newobj = new PyDialect(args, keywords);
        return newobj;
    }


    public PyDialect(PyObject[] args, String[] kwargs) {
        super();

        PyObject dialect = null;
        PyString name_obj = null;
        PyObject value_obj = null;
        ArgParser ap = null;

        System.out.println("(PyDialect) starting");

        this.quotechar = '"';
        this.delimiter = ',';
        this.escapechar = '\0';
        this.skipinitialspace = false;
        this.lineterminator = new PyString("\r\n");
        if (this.lineterminator == null) {
            throw new PyException(_csv.Error,
                "Error creating dialect - lineterminator creation failed.");
        }
        this.quoting = _csv.QUOTE_MINIMAL;
        this.doublequote = true;
        this.strict = false;

        System.out.println("(PyDialect) 1.");

        try {
            ap = new ArgParser("Dialect.init", args, kwargs, new String[] {
                "csvfile", "dialect", "delimiter", "doublequote",
               "escapechar", "lineterminator", "quotechar", "quoting",
               "skipinitialspace", "strict" });
            dialect = ap.getPyObject(1, new PyString("excel"));
        } catch (PyException ex) {
            throw Py.AttributeError(ex.value.toString());
        }

        System.out.println("(PyDialect) 2.");

        if (dialect != null) {

            // If dialect is a string, look it up in our registry
            if (dialect instanceof PyString) {
                PyObject new_dia = _csv.get_dialect_from_registry(dialect);
                if (new_dia == null) {
                    throw new PyException(_csv.Error,
                        "Error creating dialect - " +
                        "retrieval from registry failed.");
                }
                dialect = new_dia;
            }

        System.out.println("(PyDialect) 3.");

            // A class rather than an instance? Instantiate
            if (dialect instanceof PyClass) {
                PyObject new_dia;
        System.out.println("(PyDialect) 3a.");
                new_dia = dialect.__call__();
        System.out.println("(PyDialect) 3b.");
                if (new_dia == null) {
                    throw new PyException(_csv.Error, "message");
                }
                dialect = new_dia;
            }

        System.out.println("(PyDialect) 4.");

            // Make sure we finally have an instance
            PyObject dir_list = (PyList) dialect.__dir__();
            if (!(dialect instanceof PyInstance) || (dir_list == null)) {
                throw Py.TypeError("dialect must be an instance");
                    }

        System.out.println("(PyDialect) 5.");

            // And extract the attributes
            for (int i = 0; i < dir_list.__len__(); ++i) {
                String s;
                name_obj = (PyString) dir_list.__getitem__(i);
                s = name_obj.toString();
                if (s == null) {
                    throw new PyException(_csv.Error,
                        "Error initialising Dialect");
                }

                if (s.charAt(0) == '_') {
                    continue;
                }

                value_obj = dialect.__getattr__(name_obj);
                if (value_obj != null) {
                    this.__setattr__(name_obj, value_obj);
                }
            }
        }

        System.out.println("(PyDialect) 6.");

        // Check if keyword arguments were specified to override
        // specified dialect parameters.  
        // There must be a better way to do this (?) but it will do for now
        if (kwargs != null) {
            PyObject argval;
            PyString strargval;
            for (int i = 0; i < kwargs.length; ++i) {
                if (kwargs[i].equals("delimiter")) {
                    argval = ap.getPyObject(2);
                    if (!(argval instanceof PyString)) {
                        throw Py.TypeError("delimiter must be string");
                    }
                    this.__setattr__(Py.newString(kwargs[i]), argval);
                } else if (kwargs[i].equals("doublequote")) {
                    this.__setattr__(Py.newString(kwargs[i]), ap.getPyObject(3));
                } else if (kwargs[i].equals("escapechar")) {
                    this.__setattr__(Py.newString(kwargs[i]), ap.getPyObject(4));
                } else if (kwargs[i].equals("lineterminator")) {
                    this.__setattr__(Py.newString(kwargs[i]), ap.getPyObject(5));
                } else if (kwargs[i].equals("quotechar")) {
                    this.__setattr__(Py.newString(kwargs[i]), ap.getPyObject(6));
                } else if (kwargs[i].equals("quoting")) {
                    argval = ap.getPyObject(7);
                    if (!(argval instanceof PyInteger)) {
                        throw Py.TypeError("quoting must be int (0/1)");
                    }
                    this.__setattr__(Py.newString(kwargs[i]), argval);
                } else if (kwargs[i].equals("skipinitialspace")) {
                    this.__setattr__(Py.newString(kwargs[i]), ap.getPyObject(8));
                } else if (kwargs[i].equals("strict")) {
                    this.__setattr__(Py.newString(kwargs[i]), ap.getPyObject(9));
                }
            }
        }
        this.validate();
    }

    private void validate() {
/*
        if (!(this.quoting instanceof PyInteger)) {
            throw Py.TypeError("quoting must be integer (0/1)");
        }
        if (!(this.delimiter instanceof PyString)) {
            throw Py.TypeError("delimiter must be string");
        }
        if (!(this.quotechar instanceof PyString)) {
            throw Py.TypeError("quotechar must be string");
        }
*/
    }
}
