
package org.python.modules.csv;

import org.python.core.ClassDictInit;
import org.python.core.Py;
import org.python.core.PyClass;
import org.python.core.PyObject;
import org.python.core.PyString;
import org.python.core.PyStringMap;
import org.python.core.PyList;
import org.python.core.PyInteger;
import org.python.core.PyException;
import org.python.core.PyDictionary;
import org.python.core.PyNone;
import org.python.core.exceptions;
import org.python.core.ArgParser;
import org.python.modules.csv.PyReader;


public class _csv implements ClassDictInit {
    private _csv() {}

    public static void classDictInit(PyObject dict) {

        dict.__setitem__("Dialect", PyDialect.TYPE);
        dict.__setitem__("Reader", PyReader.TYPE);
        dict.__setitem__("Writer", PyWriter.TYPE);
        //dict.__setitem__("Error", _Error);
        // dict.__setitem__("excel", Pyexcel.TYPE);
        // dict.__setitem__("excel_tab", Pyexcel_tab.TYPE);
		// dict.__setitem__("DictReader", PyDictReader_tab.TYPE);
        // dict.__setitem__("DictWriter", PyDictWriter_tab.TYPE);
    }

    /**
     * Initialize the Error exception during startup
     *
     */
    public static final PyObject Error = Py.makeClass("Error",
            new PyObject[] { Py.Exception },
            Py.newJavaCode(_csv.class, "empty__init__"),
            Py.None);

    public static PyObject empty__init__(PyObject[] arg, String[] kws) {
        PyObject dict = new PyStringMap();
        dict.__setitem__("__module__", new PyString("csv"));
        return dict;
    }

    // enum QuoteSytle (_csv.c)
    public static final int QUOTE_MINIMAL = 0;

    public static final int QUOTE_ALL = 1;

    public static final int QUOTE_NONNUMERIC = 2;

    public static final int QUOTE_NONE = 3;


    // enum ParserState (_csv.c)
    static final int START_RECORD = 0;

    static final int START_FIELD = 1;

    static final int ESCAPED_CHAR = 2;

    static final int IN_FIELD = 3;

    static final int IN_QUOTED_FIELD = 4;

    static final int ESCAPE_IN_QUOTED_FIELD = 5;

    static final int QUOTE_IN_QUOTED_FIELD = 6;

    // Module version
    public static PyString __version__ = new PyString("1.0");

    // Dialect registry
    private static PyDictionary dialects = new PyDictionary();

    static PyObject get_dialect_from_registry(PyObject name_obj) {
        if (!(name_obj instanceof PyString)) {
            throw new PyException(Error,
                                  "unknown dialect. arg must be string");
        }
        PyObject dialect_obj = dialects.get(name_obj);
        if (dialect_obj instanceof PyNone) {
            throw new PyException(Error, "unknown dialect: \"" + 
                                  name_obj + "\"");
        }
        return dialect_obj;
    }

    public static void register_dialect(PyString name, PyObject dialect) {
        if (!(name instanceof PyString)) {
            throw Py.TypeError("dialect name must be a string or unicode");
        }
        if (dialect instanceof PyNone) {
            throw Py.TypeError("dialect cannot be None");
        }
        if (dialect instanceof PyClass) {
            dialect = dialect.__call__();
        }
        dialects.__setitem__(name, dialect);
        return;
    }

    public static void unregister_dialect(PyString name) {
        if (!(name instanceof PyString)) {
            throw new PyException(Error,
                                  "unknown dialect. arg must be string");
        }
        if (dialects.has_key(name)) {
            dialects.__delitem__(name);
        } else {
            throw new PyException(Error, "unknown dialect");
        }
    }

    public static PyObject get_dialect(PyString name) {
        if (!(name instanceof PyString)) {
            throw new PyException(Error,
                                  "unknown dialect. arg must be string");
        }
        if (dialects.has_key(name)) {
            PyObject dlct = dialects.__getitem__(name);
            return dlct;
        } else {
            throw new PyException(Error, "unknown dialect");
        }
    }

    public static PyObject list_dialects() {
        PyList dlcts = dialects.keys();
        return dlcts;
    }

    public static PyInteger field_size_limit() {
        PyInteger limit = new PyInteger(131072);
        return limit;
    }

    public static PyObject dialect(PyObject[] args, String[] kwargs) {

        PyDialect dlct = new PyDialect(args, kwargs);
        return dlct;
    }

    public static PyObject reader(PyObject[] args, String[] kwargs) {
        PyObject rdr = new PyReader(args, kwargs);
        return rdr;
    }

    public static PyObject writer(PyObject[] args, String[] kwargs) {
        PyObject wrtr = new PyWriter(args, kwargs);
        return wrtr;
    }

    public static String __doc__ =


"CSV parsing and writing.\n" +
"\n" +
"This module provides classes that assist in the reading and writing\n" +
"of Comma Separated Value (CSV) files, and implements the interface\n" +
"described by PEP 305.  Although many CSV files are simple to parse,\n" +
"the format is not formally defined by a stable specification and\n" +
"is subtle enough that parsing lines of a CSV file with something\n" +
"like line.split(\",\") is bound to fail.  The module supports three\n" +
"basic APIs: reading, writing, and registration of dialects.\n" +
"\n" +
"\n" +
"DIALECT REGISTRATION:\n" +
"\n" +
"Readers and writers support a dialect argument, which is a convenient\n" +
"handle on a group of settings.  When the dialect argument is a string,\n" +
"it identifies one of the dialects previously registered with the module.\n" +
"If it is a class or instance, the attributes of the argument are used as\n" +
"the settings for the reader or writer:\n" +
"\n" +
"    class excel:\n" +
"        delimiter = ','\n" +
"        quotechar = '\"'\n" +
"        escapechar = None\n" +
"        doublequote = True\n" +
"        skipinitialspace = False\n" +
"        lineterminator = '\r\n'\n" +
"        quoting = QUOTE_MINIMAL\n" +
"\n" +
"SETTINGS:\n" +
"\n" +
"    * quotechar - specifies a one-character string to use as the \n" +
"        quoting character.  It defaults to '\"'.\n" +
"    * delimiter - specifies a one-character string to use as the \n" +
"        field separator.  It defaults to ','.\n" +
"    * skipinitialspace - specifies how to interpret whitespace which\n" +
"        immediately follows a delimiter.  It defaults to False, which\n" +
"        means that whitespace immediately following a delimiter is part\n" +
"        of the following field.\n" +
"    * lineterminator -  specifies the character sequence which should \n" +
"        terminate rows.\n" +
"    * quoting - controls when quotes should be generated by the writer.\n" +
"        It can take on any of the following module constants:\n" +
"\n" +
"        csv.QUOTE_MINIMAL means only when required, for example, when a\n" +
"            field contains either the quotechar or the delimiter\n" +
"        csv.QUOTE_ALL means that quotes are always placed around fields.\n" +
"        csv.QUOTE_NONNUMERIC means that quotes are always placed around\n" +
"            fields which do not parse as integers or floating point\n" +
"            numbers.\n" +
"        csv.QUOTE_NONE means that quotes are never placed around fields.\n" +
"    * escapechar - specifies a one-character string used to escape \n" +
"        the delimiter when quoting is set to QUOTE_NONE.\n" +
"    * doublequote - controls the handling of quotes inside fields.  When\n" +
"        True, two consecutive quotes are interpreted as one during read,\n" +
"        and when writing, each quote character embedded in the data is\n" +
"        written as two quotes\n" +
"\n";


}

